<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the icon id.
 *
 * @property-read float $latitude The raw latitude value.
 * @property-read float $longitude The raw longitude value.
 */
final class Position {
	/**
	 * The latitude value of the position.
	 *
	 * @var float
	 */
	private readonly float $latitude;
	/**
	 * The longitude value of the position.
	 *
	 * @var float
	 */
	private readonly float $longitude;

	/**
	 * The constructor.
	 *
	 * @param mixed $latitude The latitude of the position.
	 * @param mixed $longitude The longitude of the position.
	 * @throws \InvalidArgumentException If the id is not numeric or less than 1.
	 */
	public function __construct( mixed $latitude, mixed $longitude ) {
		if ( ! is_float( $latitude ) && ! is_string( $latitude ) ) {
			throw new \InvalidArgumentException( 'The latitude must be a number or string.' );
		}
		if ( ! is_numeric( $latitude ) ) {
			throw new \InvalidArgumentException( 'The latitude must be numeric.' );
		}
		if ( ! is_float( $longitude ) && ! is_string( $longitude ) ) {
			throw new \InvalidArgumentException( 'The longitude must be a number or string.' );
		}
		if ( ! is_numeric( $longitude ) ) {
			throw new \InvalidArgumentException( 'The longitude must be numeric.' );
		}

		$latitude = floatval( $latitude );
		if ( $latitude < -90.0 || $latitude > 90.0 ) {
			throw new \InvalidArgumentException( 'The latitude must be in the range between -90.0 and 90.0.' );
		}

		$longitude = floatval( $longitude );
		if ( $longitude > 180.0 ) {
			$longitude = fmod( $longitude, 360.0 );
		} elseif ( $longitude < -180.0 ) {
			$longitude = abs( fmod( abs( $longitude ), 360.0 ) - 360.0 );
		}

		$this->latitude  = $latitude;
		$this->longitude = $longitude;
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'latitude' => $this->latitude,
			'longitude' => $this->longitude,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}
}
