/* global HTMLElement */

export default class Settings {
	/**
	 * @param {HTMLElement} mapElement
	 */
	constructor( mapElement ) {
		this.mapElement = mapElement;

		this.accessKey = '';
		this.boundings = undefined;
		this.location = '';
		this.locations = [];
		this.position = {
			latitude: 52.2764334,
			longitude: 10.5344468,
		};
		this.setIconOnInitialPosition = true;
		this.style = '4203';
		this.zoom = 15;
	}

	/**
	 * @param {HTMLElement} mapElement
	 */
	static async init( mapElement ) {
		const settings = new Settings( mapElement );
		return await settings.processSettings();
	}

	async processSettings() {
		await this.tryProcessSettingsByApi();
		this.tryProcessingSettingsByMapContainerAttributes();
		await this.tryProcessingLocation();

		return this;
	}

	async tryProcessingLocation() {
		if ( this.location === '' ) {
			return this;
		}

		try {
			const locationParams = new URLSearchParams();
			locationParams.set( 'text', this.location );
			const locationUrl = `https://geocode.locr.com/api/pelias/search?${ locationParams.toString() }`;
			const locationResponse = await fetch( locationUrl );
			const locationResult = await locationResponse.json();
			if ( locationResult.bbox ) {
				this.boundings = locationResult.bbox;
			}
			if (
				locationResult.features instanceof Array &&
				locationResult.features.length > 0
			) {
				if ( locationResult.features[ 0 ].geometry.type === 'Point' ) {
					this.position.latitude =
						locationResult.features[ 0 ].geometry.coordinates[ 1 ];
					this.position.longitude =
						locationResult.features[ 0 ].geometry.coordinates[ 0 ];
				}
			}
		} catch ( exc ) {
			// console.warn( 'could not find the location!', location );
		}

		return this;
	}

	async tryProcessSettingsByApi() {
		const apiFetchHandler = window.wp?.apiFetch || window.wp?.apiRequest;
		if ( ! apiFetchHandler ) {
			return this;
		}

		try {
			const data = await apiFetchHandler( {
				path: '/locrmaps/settings/all-options',
			} );

			const general = data?.general || {};
			if ( typeof general[ 'access-key' ] === 'string' ) {
				this.accessKey = general[ 'access-key' ];
			}
			if ( typeof general.location === 'string' ) {
				this.location = general.location.trim();
			}
			if ( typeof general.style === 'string' ) {
				this.style = general.style.trim();
			}

			const icon = data?.icon || {};
			if ( typeof icon[ 'set-icon-on-initial-position' ] === 'boolean' ) {
				this.setIconOnInitialPosition =
					icon[ 'set-icon-on-initial-position' ];
			}
		} catch ( exc ) {
			// ignore, because it's the first try to get the settings over the REST API!
		}

		return this;
	}

	tryProcessingSettingsByMapContainerAttributes() {
		const mapElement =
			this.mapElement || document.getElementById( 'locrmaps-map' );
		if ( ! ( mapElement instanceof HTMLElement ) ) {
			return this;
		}

		const dataAccessKeyAttr =
			mapElement.attributes.getNamedItem( 'data-access-key' );
		if ( dataAccessKeyAttr instanceof window.Attr ) {
			this.accessKey = dataAccessKeyAttr.value;
		}

		const dataLatitudeAttr =
			mapElement.attributes.getNamedItem( 'data-latitude' );
		const dataLongitudeAttr =
			mapElement.attributes.getNamedItem( 'data-longitude' );
		if (
			dataLatitudeAttr instanceof window.Attr &&
			dataLatitudeAttr.value &&
			dataLongitudeAttr instanceof window.Attr &&
			dataLongitudeAttr.value
		) {
			this.position.latitude = parseFloat( dataLatitudeAttr.value );
			this.position.longitude = parseFloat( dataLongitudeAttr.value );
		}

		const dataLocationAttr =
			mapElement.attributes.getNamedItem( 'data-location' );
		if ( dataLocationAttr instanceof window.Attr ) {
			this.location = dataLocationAttr.value;
		}

		const dataLocationsAttr =
			mapElement.attributes.getNamedItem( 'data-locations' );
		if ( dataLocationsAttr instanceof window.Attr ) {
			try {
				const parsedLocations = JSON.parse( dataLocationsAttr.value );
				if ( parsedLocations instanceof Array ) {
					this.locations = parsedLocations;
				}
			} catch ( exc ) {
				// ignore
			}
		}

		const dataSetIconOnInitialPositionAttr =
			mapElement.attributes.getNamedItem(
				'data-set-icon-on-initial-position'
			);
		if ( dataSetIconOnInitialPositionAttr instanceof window.Attr ) {
			this.setIconOnInitialPosition =
				dataSetIconOnInitialPositionAttr.value === 'true';
		}

		const dataStyleAttr =
			mapElement.attributes.getNamedItem( 'data-style' );
		if ( dataStyleAttr instanceof window.Attr && dataStyleAttr.value ) {
			this.style = dataStyleAttr.value;
		}

		const dataZoomAttr =
			mapElement.attributes.getNamedItem( 'data-zoomlevel' );
		if ( dataZoomAttr instanceof window.Attr && dataZoomAttr.value ) {
			this.zoom = parseInt( dataZoomAttr.value );
		}

		return this;
	}
}
